package uk.ac.bbsrc.babraham.SparkSpotter;
/*
 * SparkSpotter - An ImageJ plugin for finding bright sparks in movies
 * Copyright (c) 2007 Simon Andrews

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;


public class SparkSpotterPreferences extends JDialog implements ActionListener {
	
	private double foldCutoff;
	private double diffCutoff;
	private int windowSize;
	private int searchArea;
	private JTextField fold;
	private JTextField area;
	private JTextField diff;
	private JTextField window;
	private SparkSpotter_ sparkSpotter;
	private JButton analyseButton;
	

	public SparkSpotterPreferences (SparkSpotter_ sparkSpotter) {
		this.sparkSpotter = sparkSpotter;
		setTitle("Spark Spotter");

		getContentPane().setLayout(new BorderLayout());

		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.weightx = 0.5;
		gbc.weighty = 0.5;
		gbc.insets = new Insets(0,10,0,10);
		gbc.fill = GridBagConstraints.HORIZONTAL;

		mainPanel.add(new JLabel("Search area size", JLabel.LEFT),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		area = new JTextField("2",4);
		mainPanel.add(area,gbc);
		
		gbc.gridy++;
		gbc.gridx=1;
		gbc.weightx=0.5;
		
		mainPanel.add(new JLabel("SD Fold Cutoff", JLabel.LEFT),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		fold = new JTextField("5",4);
		mainPanel.add(fold,gbc);
		
		gbc.gridy++;
		gbc.gridx=1;
		gbc.weightx=0.5;
		mainPanel.add(new JLabel("Diff Cutoff", JLabel.LEFT),gbc);

		gbc.gridx=2;
		gbc.weightx=0.1;
		diff = new JTextField("50",4);
		mainPanel.add(diff,gbc);

		gbc.gridy++;
		gbc.gridx=1;
		gbc.weightx=0.5;
		mainPanel.add(new JLabel("Window Size (frames)", JLabel.LEFT),gbc);

		gbc.gridx=2;
		gbc.weightx=0.1;
		window = new JTextField("4",4);
		mainPanel.add(window,gbc);

		getContentPane().add(mainPanel,BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();

		JButton helpButton = new JButton("Help");
		helpButton.setActionCommand("help");
		helpButton.addActionListener(this);
		
		buttonPanel.add(helpButton);
		
		analyseButton = new JButton("Analyse");
		analyseButton.setActionCommand("analyse");
		analyseButton.addActionListener(this);
		
		buttonPanel.add(analyseButton);
		
		JButton closeButton = new JButton("Close");
		closeButton.setActionCommand("close");
		closeButton.addActionListener(this);
		
		buttonPanel.add(closeButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setSize(300,180);
		setLocationRelativeTo(null);
		setVisible(true);
		
		
	}

	private void startAnalysis () {
		
		// We need to check we've got sensible values in all of the
		// required fields.
		if (fold.getText().length() > 0) {
			try {
				foldCutoff = Double.parseDouble(fold.getText());
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "DS Fold Field did not contain a number", "Options error", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		else {
			foldCutoff = 0;
		}

		if (diff.getText().length() > 0) {
			try {
				diffCutoff = Double.parseDouble(diff.getText());
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Diff Field did not contain a number", "Options error", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		else {
			diffCutoff = 0;
		}

	
		if (window.getText().length() > 0) {
			try {
				windowSize = Integer.parseInt(window.getText());
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Window Size Field did not contain a number", "Options error", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		else {
			JOptionPane.showMessageDialog(this, "Window Size Field did not contain a number", "Options error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (area.getText().length() > 0) {
			try {
				searchArea = Integer.parseInt(area.getText());
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Search Area size did not contain a number", "Options error", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		else {
			searchArea = 0;
		}

		// Do some sanity checks:
		if (foldCutoff < 0 || foldCutoff > 100) {
			JOptionPane.showMessageDialog(this, "SD Fold Cutoff must be in the range 0 - 100", "Options error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (diffCutoff < 0 || diffCutoff > 10000) {
			JOptionPane.showMessageDialog(this, "Diff Cutoff must be in the range 1 - 10000", "Options error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		if (windowSize < 1 || windowSize > 1000) {
			JOptionPane.showMessageDialog(this, "Window Size must be in the range 1 - 1000", "Options error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		if (searchArea < 0 || searchArea > 100) {
			JOptionPane.showMessageDialog(this, "Search Area Size must be in the range 1 - 100", "Options error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		analyseButton.setEnabled(false);
		sparkSpotter.startAnalysis();
		
	}
	
	
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("close")){
			setVisible(false);
			dispose();
		}
		else if (e.getActionCommand().equals("analyse")) {
			startAnalysis();
		}
		else if (e.getActionCommand().equals("help")) {
			try {
				new HelpDialog(this);
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
	}

	public double foldCutoff () {
		return foldCutoff;
	}
	
	public double diffCutoff () {
		return diffCutoff;
	}
	
	public int windowSize () {
		return windowSize;
	}
	
	public int searchArea () {
		return searchArea;
	}
	
	public void analysisFinished() {
		analyseButton.setEnabled(true);
	}
	
}
